#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

module Entitlement
  #
  # Base class for all entitlement entries
  #
  class Entry
    attr_reader :file

    def initialize(modules, from = nil, to = nil, filename_string = nil, &proc)
      # rubocop:disable Style/GlobalVars
      @file            = $entitlements_file
      @warning_period  = 30
      @modules         = modules.to_a
      @from            = from
      @to              = to
      @filename_string = filename_string
      @validator       = proc
    end
    # rubocop:enable Style/GlobalVars

    def notify_if_expiring(module_name)
      return unless expiring? && valid?(module_name) && !@expire_message_printed
      @expire_message_printed = true
      Puppet.alert "*WARNING* : Only #{days_left.round} days left until your entitlement for module #{module_name} on current node is expiring. Take appropriate action."
      # rubocop:enable Metrics/LineLength
    end

    def purgable?
      @to && @to <= Date.today
    end

    # rubocop: disable Metrics/CyclomaticComplexity
    def valid?(module_name, resources = nil)
      # If the entitlement file is in the older version, do not perform check
      if @filename_string
        return false unless filename_match?
      end
      return false if @from && @from > Date.today
      return false if @to && @to <= Date.today
      return false unless matching_module(module_name)
      return true unless @validator
      @validator.call(resources)
    end
    # rubocop: enable Metrics/CyclomaticComplexity

    def inspect
      puts "Lic. File   : #{@file}"
      puts "modules     : #{@modules.join(',')}"
      puts "From date   : #{@from}" if @from
      puts "To date     : #{@to}" if @to
      puts "Purgable    : #{purgable? ? 'yes' : 'no'}"
    end

    def expiring?
      @to && @to <= (Date.today + @warning_period)
    end

    def days_left
      @to - Date.today
    end

    private

    def matching_module(name)
      @modules.any? { |m| name.match(m) }
    end

    def filename_match?
      unless @file
        raise "Entitlement file location not found."
      end
      read_file = File.basename(@file)
      check = read_file == @filename_string
      unless check
        raise "Please provide file: '#{read_file}' with originally provided filename."
      end
      check
    end
  end
end
