#
# See the file "LICENSE" for the full license governing this code.
#
# frozen_string_literal: true

# rubocop: disable Metrics/ClassLength
# rubocop: disable Lint/HandleExceptions
# rubocop: disable Metrics/AbcSize
# rubocop: disable Metrics/ParameterLists
# rubocop: disable Lint/FormatParameterMismatch
# rubocop: disable Style/RescueStandardError

require 'net/http'
require 'uri'
require 'openssl'
require 'base64'
require 'entitlement/entry'
require 'entitlement/easy_type_jwt'
require 'securerandom'

module Entitlement
  #
  # This entry allows you to specify pay as you go ruby code to be run for
  # checking entitlement validity
  #
  class PayPerUse < Entry
    HMAC_SECRET ||= '20njqjcq0c9z02x2nk4l23423n4nzxvz'.freeze

    def initialize(modules, public_key, api_url, company, node_name, &validate)
      api_url           ||= 'http://api.enterprisemodules.com/api/activation'
      @activated_modules  = []
      @activation_uri     = URI(api_url)
      @public_key         = public_key
      @company            = company
      @node_name          = node_name
      @transaction_object = transaction_object
      @cipher_key         = 'p4yasy0ugo_enterprise_modules163'
      @cipher_iv          = '0834921903420174'
      begin
        @catalog_uuid     = catalog_uuid
        @catalog_version  = catalog_version
      rescue NoMethodError
        # No catalog yet leave catalog items empty
      end
      super(modules, nil, nil, &validate)
    end

    def valid?(full_action, _resources = nil)
      fetch_module_name(full_action)
      Puppet.debug 'Found a pay as you go entitlement...'
      all_checks?
    end

    def all_checks?
      local_validated? ? true : validate_via_api
    end

    def inspect
      super
      puts 'Pay as you go: <some custom checks>'
    end

    private

    def local_validated?
      activation_registered? || @transaction_object.nil?
    end

    def activation_registered?
      @activated_modules.include?(@module_name)
    end

    def register_activation
      @activated_modules << @module_name
    end

    def validate_via_api
      res       = api_response
      decoded   = EasyTypeJWT.decode(res['authorization'][7..-1], HMAC_SECRET, 'HS256')
      json_body = JSON.parse(res.body)
      jwt_token_checks(decoded, json_body)

      return false unless activated?(json_body)
      Puppet.notice "Successful API validation. \n" \
                    "1 activation charged to your #{@company} account.\n" \
                    'Visit https://www.enterprisemodules.com/shop/account to ' \
                    "check your status.\n"
      register_activation
      true
    rescue
      Puppet.debug "Enterprisemodules response checks failes \n"
      return false
    end

    def jwt_token_checks(decoded, json_body)
      return jwt_error unless json_body['data'] == decoded[0]['data']
      return jwt_error unless decoded[0]['iss'] == 'easy_type'
      return jwt_error unless decoded[0]['sub'] == 31_268_741_268_743
      return jwt_error unless decoded[0]['iat'] <= Time.now.to_i
      return jwt_error unless decoded[0]['exp'] >= Time.now.to_i
      true
    end

    def jwt_error
      Puppet.debug "Entrprisemodules API message check failed. \n"
      false
    end

    def activated?(json_response)
      return false if json_response['data'] == false
      serialized_message = decryptor(json_response['data'])
      message = JSON.parse(serialized_message)
      matching_json_content?(message)
    end

    def matching_json_content?(message)
      message['catalog_uuid'] == @catalog_uuid && message['activated']
    end

    def api_response
      http_client.request(request_content)
    rescue
      Puppet.debug 'Calling Enterprisemodules API failed.'
      return false
    end

    def http_client
      Net::HTTP.new(@activation_uri.host, @activation_uri.port)
    end

    def request_content
      req = Net::HTTP::Post.new(@activation_uri.path,
                                'Content-Type'  => 'application/json',
                                'Authorization' => "Bearer #{token}")
      req.body = encrypted_request.to_json
      req
    end

    def token
      payload       = encrypted_request
      payload[:iat] = Time.now.to_i - (4 * 60) # Set -4 minutes issued time
      payload[:exp] = payload[:iat] + (10 * 60) # Set expiration to 10 minutes
      payload[:iss] = 'easy_type'
      payload[:sub] = '31268741268743'
      EasyTypeJWT.encode(payload, HMAC_SECRET, 'HS256')
    end

    def encrypted_request
      encoded_message = encryptor(serialized_json)
      { 'data' => encoded_message }
    end

    def serialized_json
      { 'public_key'      => @public_key,
        'module_name'     => @module_name,
        'node_name'       => @node_name,
        'catalog_uuid'    => catalog_uuid,
        'catalog_version' => catalog_version.to_s }.to_json
    end

    def transaction_object
      ObjectSpace.each_object.find do |obj|
        transaction?(obj)
      end
    end

    def transaction?(obj)
      obj.class == Puppet::Transaction
    rescue
      false
    end

    def catalog_uuid
      obj = @transaction_object.catalog.instance_variable_get(:@catalog_uuid)
      obj ||= @transaction_object.catalog.instance_variable_set(:@catalog_uuid, uuid)
      obj
    end

    # Method ported from source of Ruby 1.9.3 to 1.8.7, needed for validation
    def uuid
      ary = SecureRandom.random_bytes(16).unpack('NnnnnN')
      ary[2] = (ary[2] & 0x0fff) | 0x4000
      ary[3] = (ary[3] & 0x3fff) | 0x8000
      format('%08x-%04x-%04x-%04x-%04x%08x', ary)
    end

    def catalog_version
      @transaction_object.catalog.version.to_s
    end

    def fetch_module_name(full_action)
      @module_name = full_action.split('#')[0]
    end

    def encryptor(data)
      cipher = OpenSSL::Cipher.new('aes-256-cbc')
      cipher.encrypt
      cipher = configure_cipher(cipher)
      encrypted = cipher.update(data) + cipher.final
      Base64.encode64(encrypted)
    end

    def decryptor(data)
      byte_data = Base64.decode64(data)
      decipher = OpenSSL::Cipher.new('aes-256-cbc')
      decipher = configure_cipher(decipher)
      decipher.update(byte_data) + decipher.final
    rescue
      Puppet.debug 'Invalid Enterprisemodules server message'
      return false
    end

    def configure_cipher(cipher_object)
      cipher_object.key = @cipher_key
      cipher_object.iv = @cipher_iv
      cipher_object
    end
  end
end
# rubocop: enable Metrics/ClassLength
# rubocop: enable Lint/HandleExceptions
# rubocop: enable Metrics/AbcSize
# rubocop: enable Metrics/ParameterLists
# rubocop: enable Lint/FormatParameterMismatch
# rubocop: enable Style/RescueStandardError
